﻿using System;
using System.ComponentModel.Design;
using System.Runtime.InteropServices;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using NerdWare.Crm2011VsTools.Controls;
using NerdWare.Crm2011VsTools.Globals;

namespace NerdWare.Crm2011VsTools.ToolWindows
{
    /// <summary>
    /// This class implements the tool window exposed by this package and hosts a user control.
    ///
    /// In Visual Studio tool windows are composed of a frame (implemented by the shell) and a pane, 
    /// usually implemented by the package implementer.
    ///
    /// This class derives from the ToolWindowPane class provided from the MPF in order to use its 
    /// implementation of the IVsUIElementPane interface.
    /// </summary>
    [Guid(GuidList.ScriptInjectorToolWindowPersistanceString)]
    public class ScriptInjectorToolWindow : ToolWindowPane
    {
        /// <summary>
        /// Standard constructor for the tool window.
        /// </summary>
        public ScriptInjectorToolWindow() :
            base(null)
        {
            // Set the window title reading it from the resources.
            this.Caption = Resources.ToolWindowTitle;
            // Set the image that will appear on the tab of the window frame
            // when docked with an other window
            // The resource ID correspond to the one defined in the resx file
            // while the Index is the offset in the bitmap strip. Each image in
            // the strip being 16x16.
            this.BitmapResourceID = 500;
            this.BitmapIndex = 0;

            this.ToolBar = new CommandID(GuidList.Crm2011VsToolsCmdSet, PkgCmdIDList.ScriptInjectorToolbarID);
            this.ToolBarLocation = (int) VSTWT_LOCATION.VSTWT_TOP;

            Mcs = GetService(typeof (IMenuCommandService)) as OleMenuCommandService;

            // This is the user control hosted by the tool window; Note that, even if this class implements IDisposable,
            // we are not calling Dispose on this object. This is because ToolWindowPane calls Dispose on 
            // the object returned by the Content property.
            base.Content = new ScriptInjectorControl(this);
        }

        internal readonly OleMenuCommandService Mcs;

        internal new object GetService(Type type)
        {
            return base.GetService(type);
        }
    }
}
