﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using NerdWare.Crm2011VsTools.Core.Crm;

namespace NerdWare.Crm2011VsTools.Tests.Core.Crm
{
    [TestClass]
    public class DiscoverCrmBaseTest
    {
        private readonly Type _iDiscoverCrmBase = typeof(DiscoverCrmBase);

        #region Specification Tests
        [TestMethod]
        public void IsAPublicAbstract()
        {
            Assert.IsTrue(_iDiscoverCrmBase.IsClass, "DiscoverCrmBase need to be a Class!");
            Assert.IsTrue(_iDiscoverCrmBase.IsAbstract, "DiscoverCrmBase need to be Abstract!");
            Assert.IsTrue(_iDiscoverCrmBase.IsPublic, "DiscoverCrmBase need to be public!");
        }
        #endregion

        #region Functional Tests
        [TestMethod]
        public void CanBeInitialized()
        {
            TestDiscoverCrmBase testDiscoverCrmBase = new TestDiscoverCrmBase();
            Assert.IsNotNull(testDiscoverCrmBase);
        }

        [TestMethod]
        public void CrmApplicationDescriptorCanBeSet()
        {
            CrmApplicationDescriptor crmApplicationDescriptor = new CrmApplicationDescriptor(CrmVersionEnum.Crm2011, "testserver", 5555);
            TestDiscoverCrmBase testDiscoverCrmBase = new TestDiscoverCrmBase();
            testDiscoverCrmBase.SetApplicationDescriptor(crmApplicationDescriptor);

            Assert.AreSame(crmApplicationDescriptor, testDiscoverCrmBase.CrmApplicationDescriptor);
        }

        [TestMethod]
        public void OrgDetailsCanBeQueried()
        {
            OrgDetail testOrgDetail = new OrgDetail();

            TestDiscoverCrmBase testDiscoverCrmBase = new TestDiscoverCrmBase();
            testDiscoverCrmBase.AddOrgDetail(testOrgDetail);

            Assert.IsNotNull(testDiscoverCrmBase.Organizations);
            Assert.AreEqual(1, testDiscoverCrmBase.Organizations.Count);
            Assert.AreSame(testOrgDetail, testDiscoverCrmBase.Organizations[0]);
        }

        [TestMethod]
        public void DiscoverCanBeExecuted()
        {
            TestDiscoverCrmBase testDiscoverCrmBase = new TestDiscoverCrmBase();
            
            testDiscoverCrmBase.Discover();
        }
        #endregion
    }


    /// <summary>
    /// TestClass to create basic UnitTest on DiscoverCrmBase
    /// </summary>
    class TestDiscoverCrmBase: DiscoverCrmBase
    {
        #region Overrides of DiscoverCrmBase
        public override void Discover() {}
        #endregion

        public void AddOrgDetail(OrgDetail orgDetail)
        {
            base.Organizations.Add(orgDetail);
        }
    }
}
